// RunDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TAgent.h"
#include "RunDlg.h"
#include "stack.h"
#include "Way.h"
#include "Set.h"
#include <string.h>
#include <search.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Start Global Declarations

struct ThreadParam {     //  Structure Definition
		UINT Number;
		CWay *SearchDef;
		void *FromFile;				//  if <> NULL, search from File.
		char MapName[NAMELENGTH+1];
		class CTAgentView *pView;
		class CRunDlg *pDlg;
};
struct ThreadParam gThrParam[MAX_THREADS];
CEvent gDeleteSearch[MAX_THREADS];
CEvent gRestartSearch[MAX_THREADS];
CEvent gSaveSearch[MAX_THREADS];
CString gFileName;

UINT SearchThread(LPVOID pParam){
	struct ThreadParam *pThreadParam = (struct ThreadParam *) pParam;
	CTAgentDoc *pDoc = pThreadParam->pView->GetDocument();
	CStack Stack;
	CWay *pSolution;
	BOOL Found;

	CSet *CitiesToVisit = new CSet(pDoc->MapHeader.NumberOfCities);
	CitiesToVisit->AddPathToSet(*pThreadParam->SearchDef);

	CWay Solution(pThreadParam->SearchDef->GetFirstCity()
				 ,pThreadParam->SearchDef->GetLastCity()
				 ,pThreadParam->SearchDef->GetNumberOfCities(), FALSE);
	CWay Way(pThreadParam->SearchDef->GetFirstCity()
			,pThreadParam->SearchDef->GetLastCity()
			,pThreadParam->SearchDef->GetNumberOfCities(), FALSE);

	CWay *pWay = new CWay(pThreadParam->SearchDef->GetFirstCity()
						 ,pThreadParam->SearchDef->GetLastCity()
						 ,pThreadParam->SearchDef->GetNumberOfCities(), FALSE);

	Solution.SetSum(0xFFFFFFFFL);				//  Init Solution Total Miles / km

	//  BEGIN - Branch and Bound Alogarithem
	if(pThreadParam->FromFile)					//  if Search was loaded from File
	   Stack.LoadStack(pThreadParam->FromFile, *pThreadParam->SearchDef, &Solution);//  Load Stack Info, and dispose Memory.
	   else Stack.Push(pWay);

#ifdef _DEBUG
		   for(long k = 0; k < pWay->GetNumberOfCities(); k++)	   //  Remove After Debug
			   TRACE("RunDlg.cpp ->: City %ld = %ld\n", k, pWay->GetCity(k));
#endif	
	while(Stack.IsNOTEmpty()){					//  While Stack is NOT Empty
	  do{
		  //  Test for a DELETE event
		  if(::WaitForSingleObject(gDeleteSearch[pThreadParam->Number],0) == WAIT_OBJECT_0){
			 Stack.DisposeStack();
			 delete CitiesToVisit;
			 delete pThreadParam->SearchDef;
		     return 0;
		  }									//  End of Delete Process;

		  //  Test for a RESTART event
		  if(::WaitForSingleObject(gRestartSearch[pThreadParam->Number],0) == WAIT_OBJECT_0){
			 Stack.DisposeStack();
			 pWay = new CWay(pThreadParam->SearchDef->GetFirstCity()
							,pThreadParam->SearchDef->GetLastCity()
							,pThreadParam->SearchDef->GetNumberOfCities(), FALSE);

			 Solution.SetSum(0xFFFFFFFFL);				//  Init Solution Total Miles / km
			 Stack.Push(pWay);							//  Push first Path
		  }									//  End of Restart process

		  //  Test for a SAVE event
		  if(::WaitForSingleObject(gSaveSearch[pThreadParam->Number],0) == WAIT_OBJECT_0){
			 void *SaveInfo = Stack.SaveSearchInfo(pThreadParam->Number, pThreadParam->MapName, pThreadParam->SearchDef, &Solution);
			 pThreadParam->pDlg->SendMessage(WM_SAVE_SEARCH,ID_THREAD, (long) SaveInfo);
		  }									//  End of Save process

		  pWay = Stack.Pop();
		  Way = *pWay;						//  Researve pWay fields
		  delete pWay;						//  Release Memeory   (DISPOSE(pWay))
#ifdef _DEBUG
		   for(long k = 0; k < Way.GetNumberOfCities(); k++)	   //  Remove After Debug
			   TRACE("RunDlg.cpp ->: City %ld = %ld\n", k, Way.GetCity(k));
#endif		
		  if(Way.GetSum() < Solution.GetSum())
			 Found = Stack.PushSons(&Way, *CitiesToVisit, pDoc);
		     else Found = FALSE;
	  }while((!Found) && (Stack.GetTop())); //  Loop runs UNTIL((FOUND) OR (EMPTY(STACK))

	  if(Found){
		 Way.AddToSum(pDoc->GetDistance(Way.GetCity(Way.GetNextCity()), Way.GetLastCity()));
		 if(Way.GetLastCity() != Way.GetFirstCity())
			Way.AddCityToPath(Way.GetLastCity());    //  Add Last City to Path if NOT Roundtrip
		 if(Way.GetSum() <= Solution.GetSum()){
		    Solution = Way;					//  Update Current Solution, and POST it.
		    pSolution = new CWay(Solution);	//  The new Solution is deleted in CTAgentView::OnSolutionReady
			if(IsWindow(pThreadParam->pDlg->GetSafeHwnd())){
			   pThreadParam->pDlg->PostMessage(WM_GET_TIME_STR, 0, (long) pSolution->GetstrTime());
		       pThreadParam->pDlg->PostMessage(WM_DISPATCH_SOLUTION, 0,(long) pSolution);
			}
		 }
	  }									//  End - if(Found)
	}									//  End - while(!(IsEmpty(Stack))
	//  END - Branch and Bound Alogarithem

	//  Sending Optimal Solution
	pSolution = new CWay(Solution);		//  The new Solution is deleted in CTAgentView::OnSolutionReady
	pSolution->SetOptimal(TRUE);
	if(IsWindow(pThreadParam->pDlg->GetSafeHwnd())){
	   pThreadParam->pDlg->PostMessage(WM_GET_TIME_STR, 0, (long) pSolution->GetstrTime());
	   pThreadParam->pView->PostMessage(WM_SOLUTION_READY, TRUE,(long) pSolution);
	   }

	
	//  Cleanup after  BandB  was completed
	//  Clear singnals:
	::WaitForSingleObject(gDeleteSearch[pThreadParam->Number],0);
	::WaitForSingleObject(gRestartSearch[pThreadParam->Number],0);
	::WaitForSingleObject(gSaveSearch[pThreadParam->Number],0);

	delete CitiesToVisit;
	delete pThreadParam->SearchDef;

	if(IsWindow(pThreadParam->pView->GetSafeHwnd()))
	   pThreadParam->pView->PostMessage(WM_DELETE_SEARCH,ID_THREAD, (long) pThreadParam->Number);

	return 0;
}        //  End of SearchThread Function		
// End Global Declarations
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CRunDlg dialog


CRunDlg::CRunDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRunDlg::IDD, pParent)
{
	m_pView = NULL;
	//{{AFX_DATA_INIT(CRunDlg)
	m_nImprove = 0;
	//}}AFX_DATA_INIT
}

CRunDlg::CRunDlg(CTAgentView *pView){    //  Modeless constructor
	m_pView = pView;
	m_nSearchNumber = NOT_DEFINED;
	m_nImprove = 0;
	m_nBestPostedSum = 0xFFFFFFFFL;
	FileName.Empty();  //  Empty FileName String
	m_pThread = NULL;  //  Thread Has not Started yet
}

void CRunDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRunDlg)
	DDX_Text(pDX, IDC_IMPROVEMENT, m_nImprove);
	//}}AFX_DATA_MAP
}

BOOL CRunDlg::Create(UINT Number, CWay *Request){
//  Trying to make windows arrange properly.
//  If does no work, retain orginal routine by delelting remarks.
//  If works, Add under View menu item:  Horozental, Vertical, User
//	for Three diffrent arrangemants,
//	RECT WDim;
	m_pThread = NULL;
	m_nSearchNumber = Number;
	m_nMinutes = m_nSeconds = TimerAvail = m_nTCounter = 0;
	CWnd *TextWindow;


	BOOL Created = CDialog::Create(CRunDlg::IDD);

	if(Created){
	   CTAgentDoc *pDoc = m_pView->GetDocument();
	   if(Request->GetFirstCity() == Request->GetLastCity())
		  sprintf(MsgBuffer, "Search: Roundtrip, %s, %d cities", pDoc->Cities[Request->GetFirstCity()].Name, Request->GetNumberOfCities());
	      else sprintf(MsgBuffer, "Search: %s to %s", pDoc->Cities[Request->GetFirstCity()].Name, pDoc->Cities[Request->GetLastCity()].Name);
	   SetWindowText(MsgBuffer);		//  Set Search caption
	   if((m_nSearchNumber < 0) || (m_nSearchNumber > MAX_THREADS))
	      strcpy(MsgBuffer, "Search Was Not Defined");
	      else sprintf(MsgBuffer, "Search Number: %d", m_nSearchNumber +1);
	   TextWindow = (CWnd *) GetDlgItem(IDC_SEARCH_NUMBER);
	   TextWindow->SetWindowText(MsgBuffer);
	   TextWindow = (CWnd *) GetDlgItem(IDC_FROM_CITY);
	   sprintf(MsgBuffer, "From: %s", pDoc->Cities[Request->GetFirstCity()].Name);
	   TextWindow->SetWindowText(MsgBuffer);

	   TextWindow = (CWnd *) GetDlgItem(IDC_TO_CITY);
	   sprintf(MsgBuffer, "To: %s", pDoc->Cities[Request->GetLastCity()].Name);
	   TextWindow->SetWindowText(MsgBuffer);

	   TextWindow = (CWnd *) GetDlgItem(IDC_SEARCH_TIME);
	   TimerAvail = SetTimer(m_nSearchNumber, TIMER_PERIOD, NULL);
	   if(TimerAvail)
	      strcpy(MsgBuffer, "Search Time 00:00");
	      else strcpy(MsgBuffer, "No Timer Assigned");
	   TextWindow->SetWindowText(MsgBuffer);
	}			//  End if(Created)...
	

	return Created;
}

BEGIN_MESSAGE_MAP(CRunDlg, CDialog)
	ON_MESSAGE(WM_GET_TIME_STR,OnGetTimeStr)
	ON_MESSAGE(WM_DISPATCH_SOLUTION,OnDispatchSolution)
	ON_MESSAGE(WM_SAVE_SEARCH,OnSaveSearch)
	ON_MESSAGE(WM_SUSPEND_SEARCH_FROM_MENU, OnSuspendSearchFromMenu)
	ON_MESSAGE(WM_RESUME_SEARCH_FROM_MENU, OnResumeSearchFromMenu)
	ON_MESSAGE(WM_RESTART_SEARCH_FROM_MENU, OnRestartSearchFromMenu)
	ON_MESSAGE(WM_DELETE_SEARCH_FROM_MENU, OnDeleteSearchFromMenu)
//	ON_MESSAGE(WM_SAVE_SEARCH_FROM_MENU, OnSaveSearchFromMenu)
	//{{AFX_MSG_MAP(CRunDlg)
	ON_BN_CLICKED(ID_SUSPEND, OnSuspend)
	ON_BN_CLICKED(ID_RESUME, OnResume)
	ON_BN_CLICKED(ID_DELETE, OnDelete)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_OPTIMAL_SOLUTION, OnOptimalSolution)
	ON_BN_CLICKED(IDC_ALL_SOLUTIONS, OnAllSolutions)
	ON_EN_KILLFOCUS(IDC_IMPROVEMENT, OnKillfocusImprovement)
	ON_WM_TIMER()
	ON_BN_CLICKED(ID_RESTART, OnRestart)
	ON_BN_CLICKED(ID_SAVE, OnSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRunDlg Functions

void CRunDlg::StartSearchThread(CWay *SearchDef, void *pStack)
{
	gThrParam[m_nSearchNumber].Number = m_nSearchNumber;
	gThrParam[m_nSearchNumber].SearchDef = SearchDef;
	gThrParam[m_nSearchNumber].FromFile = pStack;
	gThrParam[m_nSearchNumber].pView = m_pView;
	strcpy(gThrParam[m_nSearchNumber].MapName, m_pView->GetDocument()->MapHeader.MapName);
	gThrParam[m_nSearchNumber].pDlg = this;

	if(!m_pThread)
	   m_pThread = AfxBeginThread(SearchThread, (LPVOID *) &gThrParam[m_nSearchNumber],THREAD_PRIORITY_BELOW_NORMAL, 4096);		//  Thread initiated with 4K Stack Memory Space
	GetDlgItem(ID_SUSPEND)->EnableWindow(TRUE);
	GetDlgItem(ID_RESUME)->EnableWindow(FALSE);
}   //  End of StartSearchThread Function

void CRunDlg::UpdateFileName(char *FName){
	FileName.Empty();
	FileName.Format("%s", FName);
}

/////////////////////////////////////////////////////////////////////////////
// CRunDlg message handlers

void CRunDlg::OnSuspend()      //  Replace ON_OK from the Book
{
	// TODO: Add your control notification handler code here
	DWORD D;
	D = SuspendThread(m_pThread->m_hThread);
	if(D == 0xFFFFFFFF)
	   AfxMessageBox("Can't Suspend Thread !");
	else {
		   if(TimerAvail)
			  KillTimer(m_nSearchNumber);
		   TimerAvail = 0;
		   GetDlgItem(ID_SUSPEND)->EnableWindow(FALSE);
		   GetDlgItem(ID_SAVE)->EnableWindow(FALSE);
		   GetDlgItem(ID_RESUME)->EnableWindow(TRUE);   
	}
}

void CRunDlg::OnResume()     //  Replace ON_CANCEL from the Book
{
	// TODO: Add your control notification handler code here
	DWORD D;
	D = ResumeThread(m_pThread->m_hThread);
	if(D == 0xFFFFFFFF)
	   AfxMessageBox("Can't Resume Thread !");
	else {
		   if(TimerAvail)
			  KillTimer(m_nSearchNumber);
		   TimerAvail = SetTimer(m_nSearchNumber, TIMER_PERIOD, NULL);
		   GetDlgItem(ID_SUSPEND)->EnableWindow(TRUE);
		   GetDlgItem(ID_RESTART)->EnableWindow(TRUE);
		   GetDlgItem(ID_SAVE)->EnableWindow(TRUE);
		   GetDlgItem(ID_RESUME)->EnableWindow(FALSE);   
	}
}

void CRunDlg::OnDelete() 
{
	// TODO: Add your control notification handler code here

	if(m_pView != NULL){
	   if(GetDlgItem(ID_RESUME)->IsWindowEnabled())   //  Thread must be running in order to Exit
		  OnResume();

	   gDeleteSearch[m_nSearchNumber].SetEvent();
	   m_pView->PostMessage(WM_DELETE_SEARCH,ID_DELETE, m_nSearchNumber);
	   FileName.Empty();
	   }
	else CDialog::OnCancel();  //  Modal case
}

void CRunDlg::OnRestart() 
{
	// TODO: Add your control notification handler code here
	CWnd *TextWindow;

	KillTimer(m_nSearchNumber);
	m_nMinutes = m_nSeconds = TimerAvail = m_nTCounter = 0;
	m_nBestPostedSum = 0xFFFFFFFFL;					//  No Solutions were posted yet (after Restart)

	TextWindow = (CWnd *) GetDlgItem(IDC_SEARCH_TIME);
	TimerAvail = SetTimer(m_nSearchNumber, TIMER_PERIOD, NULL);
	if(TimerAvail)
	   strcpy(MsgBuffer, "Search Time 00:00");
	   else strcpy(MsgBuffer, "No Timer Assigned");
	TextWindow->SetWindowText(MsgBuffer);

	if((GetDlgItem(ID_RESUME))->IsWindowEnabled())  //  Search is Suspended
	   OnResume();
	   
	GetDlgItem(ID_SUSPEND)->EnableWindow(TRUE);
	GetDlgItem(ID_RESTART)->EnableWindow(TRUE);
	GetDlgItem(ID_SAVE)->EnableWindow(TRUE);
	GetDlgItem(ID_RESUME)->EnableWindow(FALSE);

	gRestartSearch[m_nSearchNumber].SetEvent();
}

void CRunDlg::OnSave() 
{
   gSaveSearch[m_nSearchNumber].SetEvent();
}

void CRunDlg::OnClose() 
{
	// TODO: Add your message handler code here and/or call default
	if(m_pView != NULL)
	   OnDelete();					 //  Modless case - Call OnDelete to delete Search.
	else CDialog::OnClose();         //  Modal case
	
}

LRESULT  CRunDlg::OnSuspendSearchFromMenu(WPARAM wParam, LPARAM lParam)      //  Message handler to get update from Thread
{
	// TODO: Add your control notification handler code here
	OnSuspend();
	return 0L;
}

LRESULT  CRunDlg::OnResumeSearchFromMenu(WPARAM wParam, LPARAM lParam)      //  Message handler to get update from Thread
{
	// TODO: Add your control notification handler code here
	OnResume();
	return 0L;
}

LRESULT  CRunDlg::OnRestartSearchFromMenu(WPARAM wParam, LPARAM lParam)      //  Message handler to get update from Thread
{
	// TODO: Add your control notification handler code here
	OnRestart();
	return 0L;
}

LRESULT  CRunDlg::OnDeleteSearchFromMenu(WPARAM wParam, LPARAM lParam)      //  Message handler to get update from Thread
{
	// TODO: Add your control notification handler code here
	OnDelete();
	return 0L;
}

// LRESULT  CRunDlg::OnSaveSearchFromMenu(WPARAM wParam, LPARAM lParam)      //  Message handler to get update from Thread
// {
	// TODO: Add your control notification handler code here
// 	OnSave();
// 	return 0L;
// }

BOOL CRunDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	CTAgentDoc *pDoc = m_pView->GetDocument();
	m_nImprove = pDoc->MapHeader.LowImprove;
	m_nBestPostedSum = 0xFFFFFFFFL;							//  No Solutions were posted yet
    CWnd *Improvement = (CWnd *) GetDlgItem(IDC_IMPROVEMENT);
	CheckRadioButton(IDC_OPTIMAL_SOLUTION, IDC_ALL_SOLUTIONS, IDC_ALL_SOLUTIONS);
	Improvement->EnableWindow(TRUE);
	UpdateData(FALSE);
  
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CRunDlg::OnOptimalSolution() 
{
	// TODO: Add your control notification handler code here
	CButton *Optimal = (CButton *) GetDlgItem(IDC_OPTIMAL_SOLUTION);
  CWnd *Improvement = (CWnd *) GetDlgItem(IDC_IMPROVEMENT);
  if(Optimal->GetCheck())
     Improvement->EnableWindow(FALSE);
}

void CRunDlg::OnAllSolutions() 
{
	// TODO: Add your control notification handler code here
	CButton *All = (CButton *) GetDlgItem(IDC_ALL_SOLUTIONS);
  CWnd *Improvement = (CWnd *) GetDlgItem(IDC_IMPROVEMENT);
  if(All->GetCheck())
     Improvement->EnableWindow(TRUE);
}

void CRunDlg::OnKillfocusImprovement() 
{
	//  TODO: Add your control notification handler code here
  long Temp = m_nImprove;
  CTAgentDoc *pDoc = m_pView->GetDocument();
  UpdateData(TRUE);
  if((m_nImprove < pDoc->MapHeader.LowImprove) || (m_nImprove > pDoc->MapHeader.HighImprove)){
     m_nImprove = Temp;
     UpdateData(FALSE);
    }

  //  Check m_nImprove Before   Posting  or  Inserting solution to list.
  //  When Sending the Message  WM_SOLUTION_READY  
  //  might need to add  SearchNumber  as wParam, then delete solution if:
  //  if(improvment < SearchDlg[wParam]->m_nImprove)
}

LRESULT  CRunDlg::OnGetTimeStr(WPARAM wParam, LPARAM lParam){
	char *Dest = (char *) lParam;

	if(TimerAvail){
	   char MTime[3], STime[3];

	   MTime[0] = '0';
	   if(m_nMinutes < 10)
	      itoa(m_nMinutes, &MTime[1], 10);
		  else itoa(m_nMinutes, MTime, 10);

	   STime[0] = '0';
	   if(m_nSeconds < 10)
	      itoa(m_nSeconds, &STime[1], 10);
		  else itoa(m_nSeconds, STime, 10);

	   sprintf(Dest,"%s:%s", MTime, STime);
	}

	return 0L;
}

void CRunDlg::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	m_nTCounter++;
	if(m_nTCounter == 4){
	   m_nTCounter = 0;

	   m_nSeconds++;
	   if(m_nSeconds == 60){
	      m_nMinutes++;
	      m_nSeconds  = 0;
	   }
	   if(m_nMinutes == 100)
	      m_nMinutes = 0;

	   CWnd *TextWindow = (CWnd *) GetDlgItem(IDC_SEARCH_TIME);
	   if(TimerAvail){
		  char MTime[3], STime[3];

		  MTime[0] = '0';
		  if(m_nMinutes < 10)
		     itoa(m_nMinutes, &MTime[1], 10);
		     else itoa(m_nMinutes, MTime, 10);

		  STime[0] = '0';
		  if(m_nSeconds < 10)
		     itoa(m_nSeconds, &STime[1], 10);
		     else itoa(m_nSeconds, STime, 10);

	      sprintf(MsgBuffer, "Search Time: %s:%s", MTime, STime);
	   }
	   else strcpy(MsgBuffer, "No Timer Assigned");

	   TextWindow->SetWindowText(MsgBuffer);
	}

	CDialog::OnTimer(nIDEvent);
}

LRESULT  CRunDlg::OnDispatchSolution(WPARAM wParam, LPARAM lParam){
	CWay *pSolution = (CWay *) lParam;
	BOOL Post = FALSE;

	if((m_nBestPostedSum -  pSolution->GetSum()) >= m_nImprove){
	   m_nBestPostedSum = pSolution->GetSum();
	   Post = TRUE;
	   }

	if(GetCheckedRadioButton(IDC_OPTIMAL_SOLUTION, IDC_ALL_SOLUTIONS) == IDC_OPTIMAL_SOLUTION)
	   if(pSolution->IsOptimal()) Post = TRUE;

	if(Post)
	   m_pView->PostMessage(WM_SOLUTION_READY, TRUE,(long) pSolution);
	   else delete pSolution;				//  Solution Won't be Posted

	   return 0L;
}

LRESULT CRunDlg::OnSaveSearch(WPARAM wParam,LPARAM lParam){
	static int FileNumber = 1;
	SREARCHFILEHEADER *pHeader = (SREARCHFILEHEADER *) (lParam + sizeof(long));
	FILE *fp;
	CTAgentDoc *pDoc = m_pView->GetDocument();
	if(FileName.IsEmpty())
	   FileName.Format("%c%c_SRC%d.tas"
		   ,*pDoc->Cities[pHeader->FirstCity].Name
		   ,*pDoc->Cities[pHeader->LastCity].Name
		   ,FileNumber);

	OnSuspend();
	CFileDialog FDlg(FALSE, "tas", FileName.GetBuffer(1));
	if(FDlg.DoModal() == IDOK){
	   APPFILEHEADER AppHeader;
	   strcpy(AppHeader.Vertion, pDoc->AppHeader.Vertion);
	   AppHeader.FileType = FT_SEARCH;				//  FileType = Solution.
	   pHeader->Minutes = m_nMinutes;				//  Saving Time inforamtion
	   pHeader->Seconds = m_nSeconds;
	   pHeader->TCounter = m_nTCounter;

	   FileName.Empty();
	   long *SaveInfoSize = (long *) lParam;
	   FileName = FDlg.GetPathName();

	   int Selection = IDRETRY;
	   CString Msg;
	   Msg.Empty();
	   Msg.Format("%s - Cannot Open file for writing.", FileName.GetBuffer(1));
	   while(Selection == IDRETRY){
		 Selection = IDOK;
	     if((fp = fopen(FileName.GetBuffer(1), "wb")) == NULL)	   //  Save File Header + array of Cities
	        Selection = AfxMessageBox(Msg.GetBuffer(1), MB_ABORTRETRYIGNORE);
	   }

	   if(Selection == IDOK){
	      fwrite(&AppHeader, sizeof(APPFILEHEADER), 1, fp);
	      fwrite((void *) lParam, 1, ((*SaveInfoSize)*sizeof(char)) + sizeof(long), fp);
		  FileNumber++;
		  fclose(fp);
	   }
	}															//  End if(FDlg.DoModal() == IDOK) 

	OnResume();
	char *p = (char *) lParam;
	TRACE("OnSaveSearch: Deleted MemSize Momory allocation\n");
	delete [] p;										//  Delete SaveInfo

	return 0L;
}   //  End of OnSaveSearch message handling
