// SolutionDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TAgent.h"
#include "SolutionDlg.h"
#include "Way.h"
#include "Set.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSolutionDlg dialog

CSolutionDlg::CSolutionDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSolutionDlg::IDD, pParent)
{
	m_MilesDisplayType = TRUE;
	m_pView = NULL;
	SolutionNumber = 0;
	FileName.Empty();
	m_bDisplayed = FALSE;
	//{{AFX_DATA_INIT(CSolutionDlg)
	m_MilesDisplayType = -1;
	m_SolutionDesc = _T("");
	m_SolutionTime = _T("");
	m_SolutionMap = _T("");
	//}}AFX_DATA_INIT
}

CSolutionDlg::CSolutionDlg(CTAgentView *pView, const CWay *pWay, char *MapName){  //  Modeless Constructor
	m_Solution = new CWay(*pWay);
	m_MilesDisplayType = TRUE;
	m_pView = pView;
	SolutionNumber = 0;
	m_bDisplayed = FALSE;
	m_pDoc = m_pView->GetDocument();
//	CreateSolutionDesc();
	m_SolutionMap.Format(MapName);
}

BOOL CSolutionDlg::Create(int Number){
	SolutionNumber = Number;

	return CDialog::Create(CSolutionDlg::IDD);
}

void CSolutionDlg::CreateSolutionDesc(){
	char SingleDesc[NAMELENGTH + 20];
	char Units[3];
	double factor;
	ULONG Distance;
	switch(m_pDoc->MapHeader.Mesure){
	case MESURE_MILES: if(m_MilesDisplayType){
						  strcpy(Units, "mi");
						  factor = 1.0;
					   }
					   else {
							  strcpy(Units, "km");
							  factor = 1.6;
					   }
					   break;
	case MESURE_KM:    if(m_MilesDisplayType){
						  strcpy(Units, "mi");
						  factor = 1.0/1.6;
					   }
					   else {
							  strcpy(Units, "km");
							  factor = 1;
					   }
	}					//  End of case...

	m_SolutionTime.Empty();
	sprintf(SingleDesc, "Time: %s", m_Solution->GetstrTime());
	m_SolutionTime.Format(SingleDesc);
	m_SolutionDesc.Empty();
	for(int i = 0; i < m_Solution->GetNumberOfCities()-1; i++){
		Distance = (ULONG) ((m_pDoc->GetDistance(m_Solution->GetCity(i), m_Solution->GetCity(i+1))*factor)+0.5);
		sprintf(SingleDesc, "%d. %s %ld%s\n"
			   ,i+1																	//  Number of Line
			   ,m_pDoc->Cities[m_Solution->GetCity(i)].Name		//  City Name
			   ,Distance ,Units);								//  add Distance + units
		m_SolutionDesc += SingleDesc;
	}
	if(m_Solution->GetFirstCity() == m_Solution->GetLastCity()){
	   Distance = (ULONG) ((m_pDoc->GetDistance(m_Solution->GetCity(i), m_Solution->GetLastCity())*factor)+0.5);
	   sprintf(SingleDesc, "%d. %s %ld%s\n"		//  Adding Last City
		      ,i+1								//  Number of Line
		      ,m_pDoc->Cities[m_Solution->GetCity(i)].Name	//  City Name
		      ,Distance, Units);							//  add Distance + units
	   m_SolutionDesc += SingleDesc;
	}
	else i--;

	sprintf(SingleDesc, "%d. %s\n"
		   ,i+2													//  Number of Line
		   ,m_pDoc->Cities[m_Solution->GetLastCity()].Name);	//  Name only, No Distance
	m_SolutionDesc += SingleDesc;
	Distance = (ULONG) (m_Solution->GetSum()*factor);
	sprintf(SingleDesc, "\n  Total Distance: %ld", Distance);
	if(!strcmp(Units,"mi"))
	   strcat(SingleDesc, "miles");
	   else strcat(SingleDesc, "km");
	m_SolutionDesc += SingleDesc;

	UpdateData(FALSE);
}														//  End of CreateSolutionDesc Function

void CSolutionDlg::GetDescriptionSize(CSize *Size){
	CDC *dc = GetDC();
	char *Desc = m_SolutionDesc.GetBuffer(1);
	CString strLongLine;
	int Length = m_SolutionDesc.GetLength();

	int StartLine = 0, LongStart = 0, LongEnd = 0, LongLine = 0, Lines = 0;
	for(int i = 0; i < Length; i++){
		if(Desc[i] == '\n'){     //  Find the Longest Line in the Desc Text.
		   if((i - StartLine) > LongLine){
			  LongLine = i - StartLine;
			  LongStart = StartLine;
			  LongEnd = i;
		   }
		   StartLine = i+1;
		   Lines++;
		}
	}							//  End of for(UINT i...
	strLongLine = m_SolutionDesc.Mid(LongStart,LongEnd - LongStart);

	*Size = dc->GetTextExtent(strLongLine);
	Size->cy = (Size->cy)*Lines;
	ReleaseDC(dc);
}  //  End of GetDescriptionSize Function

void CSolutionDlg::SetNumber(int Number){
	SolutionNumber = Number;
}

void CSolutionDlg::UNcheckDispaly(){
	m_bDisplayed = FALSE;
	((CButton *) GetDlgItem(ID_DISPLAY))->SetCheck(m_bDisplayed);
}

void CSolutionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSolutionDlg)
	DDX_Radio(pDX, IDC_MILES, m_MilesDisplayType);
	DDX_Text(pDX, IDC_SOLUTION_DESCRIPTION, m_SolutionDesc);
	DDX_Text(pDX, IDC_SOLUTION_TIME, m_SolutionTime);
	DDX_Text(pDX, IDC_SOLUTION_MAP, m_SolutionMap);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSolutionDlg, CDialog)
	//{{AFX_MSG_MAP(CSolutionDlg)
	ON_WM_CLOSE()
	ON_BN_CLICKED(ID_CLOSE, OnCloseClicked)
	ON_BN_CLICKED(IDC_KM, OnKm)
	ON_BN_CLICKED(IDC_MILES, OnMiles)
	ON_BN_CLICKED(ID_DISPLAY, OnDisplay)
	ON_BN_CLICKED(ID_SAVE, OnSave)
	ON_BN_CLICKED(ID_PRINT, OnPrint)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSolutionDlg message handlers
void CSolutionDlg::OnDisplay() 
{
	// TODO: Add your control notification handler code here
	if(m_bDisplayed){
	   m_bDisplayed = FALSE;
	   m_pView->PostMessage(WM_SOLUTION_DISPLAY, ID_DISPLAY, NULL);	   
	}
	else {
		   m_bDisplayed = TRUE;
		   m_pView->PostMessage(WM_SOLUTION_DISPLAY, ID_DISPLAY, (long) this);
	}

	((CButton *) GetDlgItem(ID_DISPLAY))->SetCheck(m_bDisplayed);
}

void CSolutionDlg::OnClose() 
{
	// TODO: Add your control notification handler code here
	if(m_pView != NULL){								  //  Modeless Case
	   delete m_Solution;								  //  Release memory of Solution
	   m_pView->PostMessage(WM_SOLUTION_CLOSE, ID_CLOSE, SolutionNumber);
	   }
	else CDialog::OnCancel();
}

void CSolutionDlg::OnCloseClicked() 
{
	// TODO: Add your control notification handler code here
	OnClose();
}

BOOL CSolutionDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	char Buffer[80];

	m_Solution->GetSolutionTitle(Buffer, m_pDoc);
	SetWindowText(Buffer);

	CreateSolutionDesc();

	if(m_pDoc->MapHeader.Mesure == MESURE_MILES){
	   m_MilesDisplayType = TRUE;
	   CheckRadioButton(IDC_MILES, IDC_KM, IDC_MILES);
	   }
	   else {
			  m_MilesDisplayType = FALSE;
			  CheckRadioButton(IDC_MILES, IDC_KM, IDC_KM);
			}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSolutionDlg::OnKm() 
{
	// TODO: Add your control notification handler code here
	m_MilesDisplayType = FALSE;
	CreateSolutionDesc();
	CheckRadioButton(IDC_MILES, IDC_KM, IDC_KM);
}

void CSolutionDlg::OnMiles() 
{
	// TODO: Add your control notification handler code here
	m_MilesDisplayType = TRUE;
	CreateSolutionDesc();
	CheckRadioButton(IDC_MILES, IDC_KM, IDC_MILES);
}

void CSolutionDlg::OnSave() 
{
	// TODO: Add your control notification handler code here
	static int FileNumber = 1;
	FILE *fp;
	if(FileName.IsEmpty())
	   FileName.Format("%c%c_SOL%d.tas"
		   ,*m_pDoc->Cities[m_Solution->GetFirstCity()].Name
		   ,*m_pDoc->Cities[m_Solution->GetLastCity()].Name
		   ,FileNumber);

	CFileDialog FDlg(FALSE, "tas", FileName.GetBuffer(1));
	if(FDlg.DoModal() == IDOK){
	   APPFILEHEADER AppHeader;
	   SOLFILEHEADER FileHeader;						//  Build File Header
	   strcpy(AppHeader.Vertion, m_pDoc->AppHeader.Vertion);
	   AppHeader.FileType = FT_SOLUTION;				//  FileType = Solution.
	   strcpy(FileHeader.MapName, m_pDoc->MapHeader.MapName);
	   FileHeader.FirstCity = m_Solution->GetFirstCity();
	   FileHeader.LastCity = m_Solution->GetLastCity();
	   FileHeader.NumberOfCities = m_Solution->GetNumberOfCities();
	   strcpy(FileHeader.strTime, m_Solution->GetstrTime());
	   FileHeader.Optimal = m_Solution->IsOptimal();
	   FileHeader.Sum = m_Solution->GetSum();		   //   End of FileHeader struture
	   
	   FileName.Empty();
	   FileName = FDlg.GetPathName();

	   int Selection = IDRETRY;
	   CString Msg;
	   Msg.Empty();
	   Msg.Format("%s - Cannot Open file for writing.", FileName.GetBuffer(1));
	   while(Selection == IDRETRY){
		 Selection = IDOK;
	     if((fp = fopen(FileName.GetBuffer(1), "wb")) == NULL)	   //  Save File Header + array of Cities
	        Selection = AfxMessageBox(Msg.GetBuffer(1), MB_ABORTRETRYIGNORE);
	   }

	   if(Selection == IDOK){
	      fwrite(&AppHeader, sizeof(APPFILEHEADER), 1, fp);
	      fwrite(&FileHeader, sizeof(SOLFILEHEADER), 1, fp);
		  fwrite(m_Solution->GetPath(), sizeof(long), m_Solution->GetNumberOfCities(), fp);
		  FileNumber++;
		  fclose(fp);
	   }
	}															//  End if(FDlg.DoModal() == IDOK) 
}																//  End of OnSave() Function

void CSolutionDlg::OnPrint() 
{
	// TODO: Add your control notification handler code here
	m_pView->PostMessage(WM_SOLUTION_PRINT, 0,(long) this);
}
